(defun vector-cosine (x y)
  (/ (vector-dot x y) (vector-norm x) (vector-norm y)))

(defun vector-dot (x y)
  (cond (x
         (+ (* (car x) (car y))
            (vector-dot (cdr x) (cdr y))))
        (t 0)))


;; VECTOR-NORM -- also Euclidean distance
;;
(defun vector-norm (x)
  (sqrt (float (vector-sum-elements (vector-square x)))))


(defun vector-sum-elements (x)
  (cond (x
         (+ (car x) (vector-sum-elements (cdr x))))
        (t 0)))


(defun vector-sum (a b)
  (cond (a
         (cons (+ (car a) (car b))
               (vector-sum (cdr a) (cdr b))))
        (t nil)))


(defun vector-mean (x)
  (/ (vector-sum-elements x) (length x)))


(defun vector-offset (x c)
  (cond (x
         (cons (+ (car x) c) (vector-offset (cdr x) c)))
        (t nil)))


(defun vector-difference (a b)
  (cond (a
         (cons (- (car a) (car b))
               (vector-difference (cdr a) (cdr b))))
        (t nil)))


(defun vector-divide (a b)
  (cond ((null a) nil)
         (t
          (cons (/ (car a) (car b))
                (vector-divide (cdr a) (cdr b))))))


(defun vector-scale (v c)
 (cond (v (cons (/ (car v) c) (vector-scale (cdr v) c)))
       (t nil)))


(defun vector-zero (len)
  (cond ((= 0 len) nil)
        (t
         (cons 0 (vector-zero (- len 1))))))


(defun vector-square (a)
  (cond ((null a) nil)
        (t
         (cons (* (car a) (car a))
               (vector-square (cdr a))))))


(defun vector-variance (n sum sumsqr)
  (cond (sum (cons (calc-variance
                    n (car sum) (car sumsqr))
                   (vector-variance
                    n (cdr sum) (cdr sumsqr))))
        (t nil)))
